#include "execution"
#include "msl.hpp"

void
compute_income_tax(msl::Ref head)
{
	namespace p = msl::prm;
	double revenus{};
	for (msl::RefConst ref : head.over_tax_fam()) {
		if (ref.f05())
			revenus += p::p03 * ref.x21 + ref.x78;
		if (ref.f19())
			revenus += p::p03 * ref.x38;
		if (ref.f07())
			revenus += p::p12 * ref.x80 + ref.x50 + ref.x71 + ref.x39;
		if (ref.f21())
			revenus += p::p03 * ref.x62 + ref.x36;
		if (ref.f08())
			revenus += p::p17 * ref.x37;
		if (ref.f03())
			revenus += p::p03 * ref.x09;
		if (ref.f01())
			revenus += p::p23 * ref.x99;
		if (ref.f18())
			revenus += p::p03 * ref.x76;
		if (ref.f21())
			revenus += p::p45 * ref.x20 + ref.x18;
		if (ref.f20())
			revenus += p::p03 * ref.x63 + ref.x23 + ref.x65;
		if (ref.f06())
			revenus += p::p23 * ref.x35;
		if (ref.f02())
			revenus += p::p03 * ref.x77;
		if (ref.f17())
			revenus += p::p17 * ref.x70 + ref.x89;
		if (ref.f16())
			revenus += p::p03 * ref.x54 + ref.x32 + ref.x13 + ref.x27;
		if (ref.f12())
			revenus += p::p32 * ref.x72 + ref.x16;
		if (ref.f20())
			revenus += p::p03 * ref.x43 + ref.x51 + ref.x28;
		if (ref.f24())
			revenus += p::p14 * ref.x35;
		if (ref.f10())
			revenus += p::p03 * ref.x91 + ref.x22 + ref.x86;
		if (ref.f16())
			revenus += p::p16 * ref.x25;
		if (ref.f01())
			revenus += p::p03 * ref.x61 + ref.x90 + ref.x57;
		if (ref.f05())
			revenus += p::p44 * ref.x79;
		if (ref.f10())
			revenus += p::p03 * ref.x14 + ref.x95;
		if (ref.f24())
			revenus += p::p23 * ref.x66;
		if (ref.f22())
			revenus += p::p03 * ref.x96;
		if (ref.f04())
			revenus += p::p32 * ref.x59;
		if (ref.f07())
			revenus += p::p03 * ref.x44 + ref.x07;
		if (ref.f04())
			revenus += p::p15 * ref.x92 + ref.x83 + ref.x67;
		if (ref.f21())
			revenus += p::p14 * ref.x33 + ref.x52 + ref.x84 + ref.x34 + ref.x97;
		if (ref.f15())
			revenus += p::p03 * ref.x08;
		if (ref.f24())
			revenus += p::p16 * ref.x82 + ref.x48;
		if (ref.f07())
			revenus += p::p03 * ref.x17;
		if (ref.f08())
			revenus += p::p23 * ref.x64 + ref.x58;
		if (ref.f19())
			revenus += p::p03 * ref.x40 + ref.x60 + ref.x30 + ref.x55;
		if (ref.f10())
			revenus += p::p43 * ref.x56;
		if (ref.f02())
			revenus += p::p03 * ref.x74 + ref.x13 + ref.x31 + ref.x29;
		if (ref.f11())
			revenus += p::p14 * ref.x21 + ref.x88;
		if (ref.f23())
			revenus += p::p03 * ref.x69;
		if (ref.f23())
			revenus += p::p07 * ref.x46;
		if (ref.f06())
			revenus += p::p03 * ref.x94 + ref.x47 + ref.x16;
		if (ref.f13())
			revenus += p::p36 * ref.x41 + ref.x26;
		if (ref.f18())
			revenus += p::p03 * ref.x15;
		if (ref.f09())
			revenus += p::p14 * ref.x19;
		if (ref.f09())
			revenus += p::p03 * ref.x73 + ref.x10 + ref.x24 + ref.x53 + ref.x87;
		if (ref.f21())
			revenus += p::p05 * ref.x75;
		if (ref.f05())
			revenus += p::p03 * ref.x42 + ref.x68;
		if (ref.f07())
			revenus += p::p08 * ref.x12 + ref.x11 + ref.x24 + ref.x85;
		if (ref.f03())
			revenus += p::p03 * ref.x81 + ref.x49;
		if (ref.f19())
			revenus += p::p14 * ref.x93 + ref.x98;
		if (ref.f22())
			revenus += p::p03 * ref.x32 + ref.x45;
		if (ref.f14())
			revenus += p::p12 * ref.x10;
	}
	auto const business_expense_deduction{ msl::prm::business_expense_deduction_rate * revenus };
	head.income_tax = .3 * std::max(revenus - business_expense_deduction, 0.);
}

int
main()
{
	namespace v = msl::var;
	auto population{ msl::load_population("population-2024") };
	msl::load_parameters("parameters.xlsx", 2024);
	msl::load_formats("formats.xlsx");
	// Compute the amount of income tax for each tax family (baseline case).
	population.for_each_tax_fam(compute_income_tax);
	// Compute the agregated income tax revenues over tax families using the weight of the family.
	auto const tax_revenues0{ population.weighted_sum_tax_fam(v::income_tax) };
	// Compute the distribution of the burden by the standard of living of the tax families.
	auto const tab0{ population.weighted_sum_tax_fam_by(v::income_tax, v::std_living_10) };
	// Increase the rate by 10%.
	msl::prm::business_expense_deduction_rate *= 1.1;
	// Run again the computations...
	population.for_each_tax_fam(compute_income_tax);
	auto const tax_revenues1{ population.weighted_sum_tax_fam(v::income_tax) };
	auto const tab1{ population.weighted_sum_tax_fam_by(v::income_tax, v::std_living_10) };
	std::cout << "Income tax revenues (baseline)\t" << tax_revenues0 << '\n';
	std::cout << "Income tax revenues (variant)\t" << tax_revenues1 << '\n';
	// Save the distribution in a Excel workbook.
	tab0.save(tab1, "income-tax-variant.xlsx");
	return 0;
}
